@@ -1,5 +1,5 @@
 /* ksba.h - X509 library for the Aegypten project
- *      Copyright (C) 2001, 2002, 2003, 2004 g10 Code GmbH
+ *      Copyright (C) 2001, 2002, 2003, 2004, 2005 g10 Code GmbH
  *
  * This file is part of KSBA.
  *
@@ -237,6 +237,9 @@ gpg_error_t ksba_cert_get_auth_key_id (ksba_cert_t cert,
                                        ksba_sexp_t *r_keyid,
                                        ksba_name_t *r_name,
                                        ksba_sexp_t *r_serial);
+gpg_error_t ksba_cert_get_subj_key_id (ksba_cert_t cert,
+                                       int *r_crit,
+                                       ksba_sexp_t *r_keyid);
 gpg_error_t ksba_cert_get_authority_info_access (ksba_cert_t cert, int idx,
                                                  char **r_method,
                                                  ksba_name_t *r_location);
@@ -258,7 +261,7 @@ gpg_error_t ksba_cms_build (ksba_cms_t cms, ksba_stop_reason_t *r_stopreason);
 
 ksba_content_type_t ksba_cms_get_content_type (ksba_cms_t cms, int what);
 const char *ksba_cms_get_content_oid (ksba_cms_t cms, int what);
-gpg_error_t ksba_cms_get_content_enc_iv (ksba_cms_t cms, unsigned char *iv,
+gpg_error_t ksba_cms_get_content_enc_iv (ksba_cms_t cms, void *iv,
                                          size_t maxivlen, size_t *ivlen);
 const char *ksba_cms_get_digest_algo_list (ksba_cms_t cms, int idx);
 gpg_error_t ksba_cms_get_issuer_serial (ksba_cms_t cms, int idx,
@@ -291,7 +294,7 @@ gpg_error_t ksba_cms_add_smime_capability (ksba_cms_t cms, const char *oid,
                                            const unsigned char *der,
                                            size_t derlen);
 gpg_error_t ksba_cms_set_message_digest (ksba_cms_t cms, int idx,
-                                         const char *digest,
+                                         const unsigned char *digest,
                                          size_t digest_len);
 gpg_error_t ksba_cms_set_signing_time (ksba_cms_t cms, int idx,
                                        const ksba_isotime_t sigtime);
@@ -300,7 +303,7 @@ gpg_error_t ksba_cms_set_sig_val (ksba_cms_t cms,
 
 gpg_error_t ksba_cms_set_content_enc_algo (ksba_cms_t cms,
                                            const char *oid,
-                                           const unsigned char *iv,
+                                           const void *iv,
                                            size_t ivlen);
 gpg_error_t ksba_cms_add_recipient (ksba_cms_t cms, ksba_cert_t cert);
 gpg_error_t ksba_cms_set_enc_val (ksba_cms_t cms,
@@ -397,7 +400,7 @@ gpg_error_t ksba_certreq_set_public_key (ksba_certreq_t cr,
                                          ksba_const_sexp_t key);
 gpg_error_t ksba_certreq_add_extension (ksba_certreq_t cr,
                                         const char *oid, int is_crit,
-                                        const unsigned char *der,
+                                        const void *der,
                                         size_t derlen);
 gpg_error_t ksba_certreq_set_sig_val (ksba_certreq_t cr,
                                       ksba_const_sexp_t sigval);
