@@ -1,5 +1,5 @@
 /* cert-basic.c - basic test for the certificate management.
- *      Copyright (C) 2001, 2002, 2004 g10 Code GmbH
+ *      Copyright (C) 2001, 2002, 2004, 2005 g10 Code GmbH
  *
  * This file is part of KSBA.
  *
@@ -62,6 +62,20 @@ xmalloc (size_t n)
 }
 
 
+void
+print_hex (const unsigned char *p, size_t n)
+{
+  if (!p)
+    fputs ("none", stdout);
+  else
+    {
+      for (; n; n--, p++)
+        printf ("%02X", *p);
+    }
+}
+
+
+
 static void
 print_sexp (ksba_const_sexp_t p)
 {
@@ -228,6 +242,7 @@ list_extensions (ksba_cert_t cert)
   char *string, *p;
   ksba_name_t name1, name2;
   ksba_sexp_t serial;
+  ksba_sexp_t keyid;
 
   for (idx=0; !(err=ksba_cert_get_extension (cert, idx,
                                              &oid, &crit, &off, &len));idx++)
@@ -245,23 +260,49 @@ list_extensions (ksba_cert_t cert)
       errorcount++;
     }
 
-  /* authorityKeyIdentifier */
-  err = ksba_cert_get_auth_key_id (cert, NULL, &name1, &serial);
+  /* subjectKeyIdentifier */
+  err = ksba_cert_get_subj_key_id (cert, NULL, &keyid);
   if (!err || gpg_err_code (err) == GPG_ERR_NO_DATA)
     {
-      fputs ("AuthorityKeyIdentifier: ", stdout);
+      fputs ("SubjectKeyIdentifier: ", stdout);
       if (gpg_err_code (err) == GPG_ERR_NO_DATA)
         fputs ("none", stdout);
       else
         {
-          print_names (24, name1);
-          ksba_name_release (name1);
-          fputs ("                 serial: ", stdout);
-          print_sexp (serial);
-          ksba_free (serial);
+          print_sexp (keyid);
+          ksba_free (keyid);
         }
       putchar ('\n');
     }
+
+
+  /* authorityKeyIdentifier */
+  err = ksba_cert_get_auth_key_id (cert, &keyid, &name1, &serial);
+  if (!err || gpg_err_code (err) == GPG_ERR_NO_DATA)
+    {
+      fputs ("AuthorityKeyIdentifier: ", stdout);
+      if (gpg_err_code (err) == GPG_ERR_NO_DATA)
+        fputs ("none\n", stdout);
+      else
+        {
+          if (name1)
+            {
+              print_names (24, name1);
+              ksba_name_release (name1);
+              fputs ("                serial: ", stdout);
+              print_sexp (serial);
+              ksba_free (serial);
+            }
+          putchar ('\n');
+          if (keyid)
+            {
+              fputs ("         keyIdentifier: ", stdout);
+              print_sexp (keyid);
+              ksba_free (keyid);
+              putchar ('\n');
+            }
+        }
+    }
   else
     { 
       fprintf (stderr, "%s:%d: ksba_cert_get_auth_key_id: %s\n", 
